@@ -49,6 +49,8 @@
 	(G_TYPE_INSTANCE_GET_CLASS \
 	((obj), CAMEL_TYPE_FOLDER, CamelFolderClass))
 
+#define CAMEL_TYPE_FOLDER_CHANGE_INFO (camel_folder_change_info_get_type ())
+
 /**
  * CAMEL_FOLDER_ERROR:
  *
@@ -70,6 +72,14 @@ typedef struct _CamelFolderPrivate CamelFolderPrivate;
 
 /**
  * CamelFolderError:
+ * @CAMEL_FOLDER_ERROR_INVALID: a generic error about invalid operation with the folder
+ * @CAMEL_FOLDER_ERROR_INVALID_STATE: the folder is in an invalid state
+ * @CAMEL_FOLDER_ERROR_NON_EMPTY: the folder is not empty
+ * @CAMEL_FOLDER_ERROR_NON_UID: requested UID is not a UID
+ * @CAMEL_FOLDER_ERROR_INSUFFICIENT_PERMISSION: insufficient permissions for the requested operation
+ * @CAMEL_FOLDER_ERROR_INVALID_PATH: the folder path is invalid
+ * @CAMEL_FOLDER_ERROR_INVALID_UID: requested UID is invalid/cannot be found
+ * @CAMEL_FOLDER_ERROR_SUMMARY_INVALID: the folder's summary is invalid/broken
  *
  * Since: 2.32
  **/
@@ -86,6 +96,8 @@ typedef enum {
 
 /**
  * CamelFetchType:
+ * @CAMEL_FETCH_OLD_MESSAGES: fetch old messages
+ * @CAMEL_FETCH_NEW_MESSAGES: fetch new messages
  *
  * Since: 3.4
  **/
@@ -100,6 +112,7 @@ struct _CamelFolderChangeInfo {
 	GPtrArray *uid_changed;
 	GPtrArray *uid_recent;
 
+	/*< private >*/
 	CamelFolderChangeInfoPrivate *priv;
 };
 
@@ -107,6 +120,10 @@ typedef struct _CamelFolderQuotaInfo CamelFolderQuotaInfo;
 
 /**
  * CamelFolderQuotaInfo:
+ * @name: name, aka identification, of the quota type
+ * @used: how many bytes is currently in use
+ * @total: what is the maximum quota to use
+ * @next: a reference to a follwing #CamelFolderQuotaInfo
  *
  * Since: 2.24
  **/
@@ -121,14 +138,6 @@ struct _CamelFolderQuotaInfo {
 struct _CamelFolder {
 	CamelObject parent;
 	CamelFolderPrivate *priv;
-
-	CamelFolderSummary *summary;
-
-	CamelFolderFlags folder_flags;
-	CamelMessageFlags permanent_flags;
-
-	/* Future ABI expansion */
-	gpointer later[4];
 };
 
 struct _CamelFolderClass {
@@ -136,15 +145,13 @@ struct _CamelFolderClass {
 
 	/* Non-Blocking Methods */
 	gint		(*get_message_count)	(CamelFolder *folder);
-	CamelMessageFlags
-			(*get_permanent_flags)	(CamelFolder *folder);
-	CamelMessageFlags
-			(*get_message_flags)	(CamelFolder *folder,
+	guint32		(*get_permanent_flags)	(CamelFolder *folder);
+	guint32		(*get_message_flags)	(CamelFolder *folder,
 						 const gchar *uid);
 	gboolean	(*set_message_flags)	(CamelFolder *folder,
 						 const gchar *uid,
-						 CamelMessageFlags flags,
-						 CamelMessageFlags set);
+						 guint32 mask,
+						 guint32 set);
 	gboolean	(*get_message_user_flag)(CamelFolder *folder,
 						 const gchar *uid,
 						 const gchar *name);
@@ -251,9 +258,11 @@ struct _CamelFolderClass {
 						 GPtrArray **transferred_uids,
 						 GCancellable *cancellable,
 						 GError **error);
+	void		(*prepare_content_refresh)
+						(CamelFolder *folder);
 
-	/* Reserved slots for methods. */
-	gpointer reserved_for_methods[20];
+	/* Padding for future expansion */
+	gpointer reserved_methods[20];
 
 	/* Signals */
 	void		(*changed)		(CamelFolder *folder,
@@ -261,6 +270,9 @@ struct _CamelFolderClass {
 	void		(*deleted)		(CamelFolder *folder);
 	void		(*renamed)		(CamelFolder *folder,
 						 const gchar *old_name);
+
+	/* Padding for future expansion */
+	gpointer reserved_signals[20];
 };
 
 GType		camel_folder_get_type		(void);
@@ -269,6 +281,11 @@ void		camel_folder_set_lock_async	(CamelFolder *folder,
 						 gboolean skip_folder_lock);
 struct _CamelStore *
 		camel_folder_get_parent_store	(CamelFolder *folder);
+CamelFolderSummary *
+		camel_folder_get_folder_summary	(CamelFolder *folder);
+void		camel_folder_take_folder_summary
+						(CamelFolder *folder,
+						 CamelFolderSummary *summary);
 const gchar *	camel_folder_get_full_name	(CamelFolder *folder);
 gchar *		camel_folder_dup_full_name	(CamelFolder *folder);
 void		camel_folder_set_full_name	(CamelFolder *folder,
@@ -281,17 +298,18 @@ const gchar *	camel_folder_get_description	(CamelFolder *folder);
 gchar *		camel_folder_dup_description	(CamelFolder *folder);
 void		camel_folder_set_description	(CamelFolder *folder,
 						 const gchar *description);
-CamelMessageFlags
-		camel_folder_get_permanent_flags
+guint32		camel_folder_get_flags		(CamelFolder *folder);
+void		camel_folder_set_flags		(CamelFolder *folder,
+						 guint32 folder_flags);
+guint32		camel_folder_get_permanent_flags
 						(CamelFolder *folder);
 #ifndef CAMEL_DISABLE_DEPRECATED
-CamelMessageFlags
-		camel_folder_get_message_flags	(CamelFolder *folder,
+guint32		camel_folder_get_message_flags	(CamelFolder *folder,
 						 const gchar *uid);
 gboolean	camel_folder_set_message_flags	(CamelFolder *folder,
 						 const gchar *uid,
-						 CamelMessageFlags flags,
-						 CamelMessageFlags set);
+						 guint32 mask,
+						 guint32 set);
 gboolean	camel_folder_get_message_user_flag
 						(CamelFolder *folder,
 						 const gchar *uid,
@@ -378,8 +396,6 @@ CamelFolderQuotaInfo *
 CamelFolderQuotaInfo *
 		camel_folder_quota_info_clone	(const CamelFolderQuotaInfo *info);
 void		camel_folder_quota_info_free	(CamelFolderQuotaInfo *info);
-void		camel_folder_free_nop		(CamelFolder *folder,
-						 GPtrArray *array);
 void		camel_folder_free_shallow	(CamelFolder *folder,
 						 GPtrArray *array);
 void		camel_folder_free_deep		(CamelFolder *folder,
@@ -435,6 +451,10 @@ CamelMimeMessage *
 		camel_folder_get_message_finish	(CamelFolder *folder,
 						 GAsyncResult *result,
 						 GError **error);
+CamelMimeMessage *
+		camel_folder_get_message_cached	(CamelFolder *folder,
+						 const gchar *message_uid,
+						 GCancellable *cancellable);
 CamelFolderQuotaInfo *
 		camel_folder_get_quota_info_sync
 						(CamelFolder *folder,
@@ -531,13 +551,27 @@ gboolean	camel_folder_transfer_messages_to_finish
 						 GAsyncResult *result,
 						 GPtrArray **transferred_uids,
 						 GError **error);
+void		camel_folder_prepare_content_refresh
+						(CamelFolder *folder);
 
 /* update functions for change info */
+GType		camel_folder_change_info_get_type
+						(void) G_GNUC_CONST;
 CamelFolderChangeInfo *
 		camel_folder_change_info_new	(void);
+CamelFolderChangeInfo *
+		camel_folder_change_info_copy	(CamelFolderChangeInfo *src);
 void		camel_folder_change_info_clear	(CamelFolderChangeInfo *info);
 void		camel_folder_change_info_free	(CamelFolderChangeInfo *info);
 gboolean	camel_folder_change_info_changed (CamelFolderChangeInfo *info);
+GPtrArray *	camel_folder_change_info_get_added_uids
+						(CamelFolderChangeInfo *info);
+GPtrArray *	camel_folder_change_info_get_removed_uids
+						(CamelFolderChangeInfo *info);
+GPtrArray *	camel_folder_change_info_get_changed_uids
+						(CamelFolderChangeInfo *info);
+GPtrArray *	camel_folder_change_info_get_recent_uids
+						(CamelFolderChangeInfo *info);
 
 /* for building diff's automatically */
 void		camel_folder_change_info_add_source
